@@ -1,4 +1,4 @@
-/* armor.c - Armor filter
+/* armor.c - Armor flter
  * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
  *               2007 Free Software Foundation, Inc.
  *
@@ -381,32 +381,6 @@ is_armor_header( byte *line, unsigned len )
 }
 
 
-/* Helper to parse a "KEY <keyid> FAILED <code>" line and return the
-   error code.  LINEPTR points right behind "KEY ".  */
-int
-parse_key_failed_line (const void *lineptr, unsigned int len)
-{
-  const byte *line = lineptr;
-  int code = 0;
-
-  for (; len && !spacep (line); len--, line++)
-    ;
-  for (; len && spacep (line); len--, line++)
-    ;
-  if (len > 7 && !memcmp (line, "FAILED ", 7))
-    {
-      line += 7;
-      len -= 7;
-      for (; len && digitp (line); len--, line++)
-        {
-          code *= 10;
-          code += atoi_1 (line);
-        }
-    }
-
-  return code;
-}
-
 
 /****************
  * Parse a header lines
@@ -441,9 +415,9 @@ parse_header_line( armor_filter_context_t *afx, byte *line, unsigned int len )
     if( !p || (RFC2440 && p[1]!=' ')
 	|| (!RFC2440 && p[1]!=' ' && p[1]!='\n' && p[1]!='\r'))
       {
-	log_error(_("invalid armor header: "));
-	print_string( stderr, line, len, 0 );
-	putc('\n', stderr);
+	log_error (_("invalid armor header: "));
+	es_write_sanitized (log_get_stream (), line, len, NULL, NULL);
+	log_printf ("\n");
 	return -1;
       }
 
@@ -453,8 +427,8 @@ parse_header_line( armor_filter_context_t *afx, byte *line, unsigned int len )
 
     if( opt.verbose ) {
 	log_info(_("armor header: "));
-	print_string( stderr, line, len, 0 );
-	putc('\n', stderr);
+	es_write_sanitized (log_get_stream (), line, len, NULL, NULL);
+	log_printf ("\n");
     }
 
     if( afx->in_cleartext )
@@ -479,8 +453,8 @@ parse_header_line( armor_filter_context_t *afx, byte *line, unsigned int len )
 	   signed data section is "Hash". */
 
 	log_info(_("unknown armor header: "));
-	print_string( stderr, line, len, 0 );
-	putc('\n', stderr);
+	es_write_sanitized (log_get_stream (), line, len, NULL, NULL);
+	log_printf ("\n");
       }
 
     return 1;
@@ -527,14 +501,6 @@ check_input( armor_filter_context_t *afx, IOBUF a )
     /* find the armor header */
     while(len) {
 	i = is_armor_header( line, len );
-        if (i == -1 && afx->only_keyblocks
-            && !afx->key_failed_code
-            && len > 4 && !memcmp (line, "KEY ", 4))
-          {
-            /* This is probably input from a keyserver helper and we
-               have not yet seen an error line.  */
-            afx->key_failed_code = parse_key_failed_line (line+4, len-4);
-          }
 	if( i >= 0 && !(afx->only_keyblocks && i != 1 && i != 5 && i != 6 )) {
 	    hdr_line = i;
 	    if( hdr_line == BEGIN_SIGNED_MSG_IDX ) {
@@ -675,8 +641,9 @@ fake_packet( armor_filter_context_t *afx, IOBUF a,
 		    if( type != BEGIN_SIGNATURE )
 		      {
 			log_info(_("unexpected armor: "));
-			print_string( stderr, p, n, 0 );
-			putc('\n', stderr);
+			es_write_sanitized (log_get_stream (), p, n,
+                                            NULL, NULL);
+			log_printf ("\n");
 		      }
 
 		    lastline = 1;
@@ -686,9 +653,9 @@ fake_packet( armor_filter_context_t *afx, IOBUF a,
 	    else if(!afx->not_dash_escaped)
 	      {
 		/* Bad dash-escaping. */
-		log_info(_("invalid dash escaped line: "));
-		print_string( stderr, p, n, 0 );
-		putc('\n', stderr);
+		log_info (_("invalid dash escaped line: "));
+		es_write_sanitized (log_get_stream (), p, n, NULL, NULL);
+		log_printf ("\n");
 	      }
 	  }
 
@@ -1112,7 +1079,7 @@ armor_filter( void *opaque, int control,
 	    iobuf_writestr(a,afx->eol);
 	    if (opt.emit_version)
 	      {
-		iobuf_writestr (a, "Version: GnuPG v");
+		iobuf_writestr (a, "Version: "GNUPG_NAME" v");
                 for (s=VERSION; *s && *s != '.'; s++)
                   iobuf_writebyte (a, *s);
                 if (opt.emit_version > 1 && *s)
@@ -1229,21 +1196,20 @@ armor_filter( void *opaque, int control,
 	    crc = afx->crc;
 	    idx = afx->idx;
 	    idx2 = afx->idx2;
-	    for(i=0; i < idx; i++ )
-		radbuf[i] = afx->radbuf[i];
 	    if( idx ) {
-		c = bintoasc[(*radbuf>>2)&077];
+		c = bintoasc[(afx->radbuf[0]>>2)&077];
 		iobuf_put(a, c);
 		if( idx == 1 ) {
-		    c = bintoasc[((*radbuf << 4) & 060) & 077];
+		    c = bintoasc[((afx->radbuf[0] << 4) & 060) & 077];
 		    iobuf_put(a, c);
 		    iobuf_put(a, '=');
 		    iobuf_put(a, '=');
 		}
 		else { /* 2 */
-		    c = bintoasc[(((*radbuf<<4)&060)|((radbuf[1]>>4)&017))&077];
+		    c = bintoasc[(((afx->radbuf[0]<<4)&060)
+                                  |((afx->radbuf[1]>>4)&017))&077];
 		    iobuf_put(a, c);
-		    c = bintoasc[((radbuf[1] << 2) & 074) & 077];
+		    c = bintoasc[((afx->radbuf[1] << 2) & 074) & 077];
 		    iobuf_put(a, c);
 		    iobuf_put(a, '=');
 		}
