@@ -5,7 +5,7 @@
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -14,9 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -120,6 +118,7 @@ do_uncompress( compress_filter_context_t *zfx, bz_stream *bzs,
   size_t n;
   int nread, count;
   int refill = !bzs->avail_in;
+  int eofseen = 0;
 
   if( DBG_FILTER )
     log_debug("begin bzDecompress: avail_in=%u, avail_out=%u, inbuf=%u\n",
@@ -134,12 +133,16 @@ do_uncompress( compress_filter_context_t *zfx, bz_stream *bzs,
 	    bzs->next_in = zfx->inbuf;
 	  count = zfx->inbufsize - n;
 	  nread = iobuf_read( a, zfx->inbuf + n, count );
-	  if( nread == -1 ) nread = 0;
+	  if( nread == -1 )
+            {
+              eofseen = 1;
+              nread = 0;
+            }
 	  n += nread;
 	  bzs->avail_in = n;
 	}
-
-      refill = 1;
+      if (!eofseen)
+        refill = 1;
 
       if( DBG_FILTER )
 	log_debug("enter bzDecompress: avail_in=%u, avail_out=%u\n",
@@ -153,6 +156,13 @@ do_uncompress( compress_filter_context_t *zfx, bz_stream *bzs,
 	rc = -1; /* eof */
       else if( zrc != BZ_OK && zrc != BZ_PARAM_ERROR )
 	log_fatal("bz2lib inflate problem: rc=%d\n", zrc );
+      else if (zrc == BZ_OK && eofseen
+               && !bzs->avail_in && bzs->avail_out > 0)
+        {
+          log_error ("unexpected EOF in bz2lib\n");
+          rc = GPG_ERR_BAD_DATA;
+          break;
+        }
     }
   while( bzs->avail_out && zrc != BZ_STREAM_END && zrc != BZ_PARAM_ERROR );
 
