@@ -67,20 +67,23 @@ write_record( TRUSTREC *rec )
 void
 list_trustdb( const char *username )
 {
-    TRUSTREC rec;
+  TRUSTREC rec;
 
-    init_trustdb();
-    /* for now we ignore the user ID */
-    if (1) {
-	ulong recnum;
-	int i;
+  (void)username;
 
-	printf("TrustDB: %s\n", tdbio_get_dbname() );
-	for(i=9+strlen(tdbio_get_dbname()); i > 0; i-- )
-	    putchar('-');
-	putchar('\n');
-	for(recnum=0; !tdbio_read_record( recnum, &rec, 0); recnum++ )
-	    tdbio_dump_record( &rec, stdout );
+  init_trustdb();
+  /* For now we ignore the user ID. */
+  if (1)
+    {
+      ulong recnum;
+      int i;
+
+      printf("TrustDB: %s\n", tdbio_get_dbname() );
+      for(i=9+strlen(tdbio_get_dbname()); i > 0; i-- )
+        putchar('-');
+      putchar('\n');
+      for(recnum=0; !tdbio_read_record( recnum, &rec, 0); recnum++ )
+        tdbio_dump_record( &rec, stdout );
     }
 }
 
@@ -136,15 +139,15 @@ import_ownertrust( const char *fname )
 	is_stdin = 1;
     }
     else if( !(fp = fopen( fname, "r" )) ) {
-	log_error ( _("can't open `%s': %s\n"), fname, strerror(errno) );
+	log_error ( _("can't open '%s': %s\n"), fname, strerror(errno) );
 	return;
     }
 
     if (is_secured_file (fileno (fp)))
       {
         fclose (fp);
-        errno = EPERM;
-	log_error (_("can't open `%s': %s\n"), fname, strerror(errno) );
+        gpg_err_set_errno (EPERM);
+	log_error (_("can't open '%s': %s\n"), fname, strerror(errno) );
 	return;
       }
 
@@ -155,7 +158,7 @@ import_ownertrust( const char *fname )
 	    continue;
 	n = strlen(line);
 	if( line[n-1] != '\n' ) {
-	    log_error (_("error in `%s': %s\n"), fname, _("line too long") );
+	    log_error (_("error in '%s': %s\n"), fname, _("line too long") );
 	    /* ... or last line does not have a LF */
 	    break; /* can't continue */
 	}
@@ -163,17 +166,17 @@ import_ownertrust( const char *fname )
 	    if( !hexdigitp(p) )
 		break;
 	if( *p != ':' ) {
-	    log_error (_("error in `%s': %s\n"), fname, _("colon missing") );
+	    log_error (_("error in '%s': %s\n"), fname, _("colon missing") );
 	    continue;
 	}
 	fprlen = p - line;
 	if( fprlen != 32 && fprlen != 40 ) {
-	    log_error (_("error in `%s': %s\n"),
+	    log_error (_("error in '%s': %s\n"),
                        fname, _("invalid fingerprint") );
 	    continue;
 	}
 	if( sscanf(p, ":%u:", &otrust ) != 1 ) {
-	    log_error (_("error in `%s': %s\n"),
+	    log_error (_("error in '%s': %s\n"),
                        fname, _("ownertrust value missing"));
 	    continue;
 	}
@@ -184,7 +187,7 @@ import_ownertrust( const char *fname )
 	    fpr[fprlen++] = HEXTOBIN(p[0]) * 16 + HEXTOBIN(p[1]);
 	while (fprlen < 20)
 	    fpr[fprlen++] = 0;
-        
+
 	rc = tdbio_search_trust_byfpr (fpr, &rec);
 	if( !rc ) { /* found: update */
 	    if (rec.r.trust.ownertrust != otrust)
@@ -210,14 +213,14 @@ import_ownertrust( const char *fname )
             any = 1;
 	}
 	else /* error */
-	    log_error (_("error finding trust record in `%s': %s\n"),
+	    log_error (_("error finding trust record in '%s': %s\n"),
                        fname, g10_errstr(rc));
     }
     if( ferror(fp) )
-	log_error ( _("read error in `%s': %s\n"), fname, strerror(errno) );
+	log_error ( _("read error in '%s': %s\n"), fname, strerror(errno) );
     if( !is_stdin )
 	fclose(fp);
-    
+
     if (any)
       {
         revalidation_mark ();
@@ -225,7 +228,5 @@ import_ownertrust( const char *fname )
         if (rc)
           log_error (_("trustdb: sync failed: %s\n"), g10_errstr(rc) );
       }
-    
+
 }
-
-
