@@ -74,7 +74,9 @@
 #ifdef HAVE_PTY_H
 #include <pty.h>
 #endif
+#ifdef HAVE_UTMP_H
 #include <utmp.h>
+#endif
 #include <ctype.h>
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
@@ -87,6 +89,7 @@
 #define JNLIB_NEED_LOG_LOGV
 #include "i18n.h"
 #include "../common/util.h"
+#include "../common/init.h"
 #include "mkdtemp.h"
 
 /* FIXME: Bah.  For spwq_secure_free.  */
@@ -96,10 +99,10 @@
 
 /* From simple-gettext.c.  */
 
-/* We assume to have `unsigned long int' value with at least 32 bits.  */
+/* We assume to have 'unsigned long int' value with at least 32 bits.  */
 #define HASHWORDBITS 32
 
-/* The so called `hashpjw' function by P.J. Weinberger
+/* The so called 'hashpjw' function by P.J. Weinberger
    [see Aho/Sethi/Ullman, COMPILERS: Principles, Techniques and Tools,
    1986, 1987 Bell Telephone Laboratories, Inc.]  */
 
@@ -153,9 +156,9 @@ static ARGPARSE_OPTS opts[] =
 
     { oDecrypt, "decrypt", 0, N_("decryption modus") },
     { oEncrypt, "encrypt", 0, N_("encryption modus") },
-    
+
     { 302, NULL, 0, N_("@\nOptions:\n ") },
-    
+
     { oClass, "class", 2, N_("tool class (confucius)") },
     { oProgram, "program", 2, N_("program filename") },
 
@@ -168,7 +171,7 @@ static ARGPARSE_OPTS opts[] =
 
     /* Hidden options.  */
     { oNoVerbose, "no-verbose",  0, "@" },
-    { oHomedir, "homedir", 2, "@" },   
+    { oHomedir, "homedir", 2, "@" },
     { oNoOptions, "no-options", 0, "@" },/* shortcut for --options /dev/null */
 
     {0}
@@ -197,12 +200,12 @@ my_strusage (int level)
 
   switch (level)
     {
-    case 11: p = "symcryptrun (GnuPG)";
+    case 11: p = "symcryptrun (@GNUPG@)";
       break;
     case 13: p = VERSION; break;
     case 17: p = PRINTABLE_OS_NAME; break;
-    case 19: p = _("Please report bugs to <" PACKAGE_BUGREPORT ">.\n");
-      break;
+    case 19: p = _("Please report bugs to <@EMAIL@>.\n"); break;
+
     case 1:
     case 40: p = _("Usage: symcryptrun [options] (-h for help)");
       break;
@@ -225,7 +228,7 @@ my_strusage (int level)
 /* This is in the GNU C library in unistd.h.  */
 
 #ifndef TEMP_FAILURE_RETRY
-/* Evaluate EXPRESSION, and repeat as long as it returns -1 with `errno'
+/* Evaluate EXPRESSION, and repeat as long as it returns -1 with 'errno'
    set to EINTR.  */
 
 # define TEMP_FAILURE_RETRY(expression) \
@@ -254,10 +257,10 @@ remove_file (char *name, int shred)
       if (pid == 0)
 	{
 	  /* Child.  */
-	  
+
 	  /* -f forces file to be writable, and -u unlinks it afterwards.  */
 	  char *args[] = { SHRED, "-uf", name, NULL };
-	  
+
 	  execv (SHRED, args);
 	  _exit (127);
 	}
@@ -269,11 +272,11 @@ remove_file (char *name, int shred)
       else
 	{
 	  /* Parent.  */
-	  
+
 	  if (TEMP_FAILURE_RETRY (waitpid (pid, &status, 0)) != pid)
 	    status = -1;
 	}
-      
+
       if (!WIFEXITED (status))
 	{
 	  log_error (_("%s on %s aborted with status %i\n"),
@@ -304,12 +307,18 @@ remove_file (char *name, int shred)
 static char *
 confucius_mktmpdir (void)
 {
-  char *name;
+  char *name, *p;
 
-  name = strdup ("/tmp/gpg-XXXXXX");
+  p = getenv ("TMPDIR");
+  if (!p || !*p)
+    p = "/tmp";
+  if (p[strlen (p) - 1] == '/')
+    name = xstrconcat (p, "gpg-XXXXXX", NULL);
+  else
+    name = xstrconcat (p, "/", "gpg-XXXXXX", NULL);
   if (!name || !mkdtemp (name))
     {
-      log_error (_("can't create temporary directory `%s': %s\n"),
+      log_error (_("can't create temporary directory '%s': %s\n"),
                  name?name:"", strerror (errno));
       return NULL;
     }
@@ -428,7 +437,7 @@ confucius_get_pass (const char *cacheid, int again, int *canceled)
 
   if (canceled)
     *canceled = 0;
-  
+
   orig_codeset = i18n_switchto_utf8 ();
   pw = simple_pwquery (cacheid,
                        again ? _("does not match - try again"):NULL,
@@ -446,7 +455,7 @@ confucius_get_pass (const char *cacheid, int again, int *canceled)
 	  log_info (_("cancelled\n"));
 	  if (canceled)
 	    *canceled = 1;
-	}      
+	}
     }
 
   return pw;
@@ -555,7 +564,7 @@ confucius_process (int mode, char *infile, char *outfile,
       free (args);
       return 1;
     }
-  else if (pid == 0) 
+  else if (pid == 0)
     {
       /* Child.  */
 
@@ -628,7 +637,7 @@ confucius_process (int mode, char *infile, char *outfile,
 		  close (cstderr[0]);
 		  return 1;
 		}
-	      else  
+	      else
 		{
 		  char *newline;
 
@@ -854,7 +863,7 @@ confucius_main (int mode, int argc, char *argv[])
       rmdir (tmpdir);
       return res;
     }
-  
+
   remove_file (outfile, mode == oDecrypt);
   if (infile_from_stdin)
     remove_file (infile, mode == oEncrypt);
@@ -874,7 +883,7 @@ main (int argc, char **argv)
   char **orig_argv;
   FILE *configfp = NULL;
   char *configname = NULL;
-  unsigned configlineno; 
+  unsigned configlineno;
   int mode = 0;
   int res;
   char *logfile = NULL;
@@ -885,7 +894,7 @@ main (int argc, char **argv)
 
   /* Make sure that our subsystems are ready.  */
   i18n_init();
-  init_common_subsystems ();
+  init_common_subsystems (&argc, &argv);
 
   opt.homedir = default_homedir ();
 
@@ -911,7 +920,7 @@ main (int argc, char **argv)
 
   if (default_config)
     configname = make_filename (opt.homedir, "symcryptrun.conf", NULL );
-  
+
   argc = orig_argc;
   argv = orig_argv;
   pargs.argc = &argc;
@@ -926,11 +935,11 @@ main (int argc, char **argv)
         {
           if (!default_config)
             {
-              log_error (_("option file `%s': %s\n"),
+              log_error (_("option file '%s': %s\n"),
                          configname, strerror(errno) );
               exit(1);
 	    }
-          xfree (configname); 
+          xfree (configname);
           configname = NULL;
 	}
       default_config = 0;
@@ -947,7 +956,7 @@ main (int argc, char **argv)
 	case oQuiet:     opt.quiet = 1; break;
         case oVerbose:   opt.verbose++; break;
         case oNoVerbose: opt.verbose = 0; break;
-	  
+
 	case oClass:	opt.class = pargs.r.ret_str; break;
 	case oProgram:	opt.program = pargs.r.ret_str; break;
 	case oKeyfile:	opt.keyfile = pargs.r.ret_str; break;
@@ -1001,7 +1010,7 @@ main (int argc, char **argv)
 
   /* Tell simple-pwquery about the the standard socket name.  */
   {
-    char *tmp = make_filename (opt.homedir, "S.gpg-agent", NULL);
+    char *tmp = make_filename (opt.homedir, GPG_AGENT_SOCK_NAME, NULL);
     simple_pw_set_socket (tmp);
     xfree (tmp);
   }
